#include "agent/linux/get_distro.h"

int open_file(std::string name, std::ifstream& fstream){
		fstream.open(name);
		if (fstream.is_open())
			return 1;
		else
			return 0;
}

std::string get_distro(){
	/* in Linux: distro name + version + release
	   the year is 2016 and we have 2 "unified" ways to identify distros
	   and a couple regex to find the distro name files
	   update: I used to complain about Linux, then I tried coding for
	   Windows... */

	/* current recorded distros are
	   * Ubuntu
	   * Debian
	   * Linux Comunicações
	   * Linux Educacional
	   all of which should have the lsb-release file, but I added a couple more
	   common distros just in case */

	std::ifstream release_file;
	std::string line;

	// lsb compliant
	if (open_file("/etc/lsb-release", release_file)) {
		const std::string key = "DISTRIB_DESCRIPTION=";
		std::size_t found;

		while (getline(release_file, line)) {
			found = line.find(key);
			if (found!=std::string::npos) {
				return (line.substr(found+key.length()+1,
								   // remove key and quotes at the beginning
								   line.length()-(found+key.length()+1)-1));
								   // remove quotes at the end
			}
		}
	}

	// openSuSE
	else if (open_file("/etc/os-release", release_file)){
		const std::string name_key = "NAME=";
		const std::string version_key = "VERSION=";

		std::string name;
		std::string version;

		std::size_t found_name;
		std::size_t found_version;

		while (getline(release_file, line)) {
			found_name = line.find(name_key);
			found_version = line.find(version_key);
			if (found_name < line.length()) {
				name = line.substr(found_name+name_key.length()+1,
								   // remove key and quotes at the beginning
								   line.length()-(found_name+name_key.length()+1)-1);
								   // remove quotes at the end
			}

			if (found_version < line.length()) {
				version = line.substr(found_version+version_key.length()+1,
								   // remove key and quotes at the beginning
								   line.length()-(found_version+version_key.length()+1)-1);
								   // remove quotes at the end
			}

			if (name.length() && version.length()) {
				return name + version;
			}
		}

		if (name.length()) {
			return name;
		}
		else {
			return "unknown";
		}
	}

	else if (open_file("/etc/novell-release", release_file)){
		getline(release_file, line);
		return line;
	}

	else if (open_file("/etc/sles-release", release_file)){
		getline(release_file, line);
		return line;
	}

	// fedora
	else if (open_file("/etc/redhat-release", release_file)){
		getline(release_file, line);
		return line;
	}

	// debian -- leave this at the end, otherwise debian-based distros
	//			 will find the debian_version file instead of the actual
	//			 distro name
	else if (open_file("/etc/debian-release", release_file)){
		getline(release_file, line);
		return line;
	}

	else if (open_file("/etc/debian-version", release_file)){
		getline(release_file, line);
		return line;
	}

	return "unknown";
}
