/* Copyright (C) 2004-2016 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of datasid
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 */
#include "../../include/agent/linux/datasid-conf.h"
#include "../../include/agent/linux/datasid-proxy.h"
#include "../../include/agent/linux/datasid-agent.h"
#include "../../include/agent/linux/datasid-common.h"
#include "../../include/agent/main.h"
using namespace std;

/*
 * set Functions
 */

void Common::setDefaultConf(string defaultConf, string prefix){
    this->defaultConf = prefix + defaultConf;
}

/*
 * get Functions
 */

string Common::getDefaultConf() const{
    return  this->defaultConf;
}

/*
 * fileExist Function
 * verify if a file exist
 */

bool fileExist(const string& file){
    ifstream myFile (file.c_str(), ios::in);
    if(myFile.is_open()){
        myFile.close();
        return true;
    }
    myFile.close();
    return false;
}


bool datasid_common(Conf& conf, Proxy& proxy, Agent& agent, Common& common){

    try{
        prefixIsSet(agent.getPrefix());
    }catch(const char* msg){
        cout << __DATE__ << msg << endl;
        return false;
    }

    /* Read config file and declare only valid variables */

    common.setDefaultConf("/conf/datasid-conf.json", agent.getPrefix());
    if(!fileExist(common.getDefaultConf())){
        cout << __DATE__ << "- ERROR: Config file not found. " << endl;
        return false;
    } else 
        conf = datasid_conf(agent);	

    /* Read proxy file and declare only valid variables */

    if(!fileExist(conf.getProxyConf()))
        cout << __DATE__ << "- WARNING: Proxy file not found. " << endl;
    else
        proxy = datasid_proxy(conf);

    return true;
}