/* Copyright (C) 2004-2016 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of datasid
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 */
#include "../../include/agent/linux/datasid-conf.h"
#include <map>
#include "../../include/agent/linux/datasid-agent.h"
#include "../../include/agent/main.h"
#include "../../include/agent/linux/datasid-parse.h"

using namespace std;

/*
 * set Functions
 */

void Conf::setDataDir(string data){
    this->dataDir = data;
}

void Conf::setLogDir(string log){
    this->logDir = log;
}

void Conf::setProxyConf(string p){
    this->proxyConf = p;
}

void Conf::setWebService(string web){
    this->webService = web;
}

void Conf::setDelay(int d){
    this->delay = d;
}

void Conf::setDaysToExpireLog(int d){
    this->daysToExpireLog = d;
}

void Conf::setUpdateTries(int u){
    this->updateTries = u;
}

void Conf::setUpdatTimeOut(int u){
    this->updateTimeOut = u;
}

/*
 * get Functions
 */

string Conf::getDataDir() const{
    return  this->dataDir;
}

string Conf::getLogDir() const{
    return this->logDir;
}

string Conf::getProxyConf() const{
    return this->proxyConf;
}

string Conf::getWebService() const{
    return this->webService;
}

int Conf::getDelay() const{
    return this->delay;
}

int Conf::getDaysToExpireLog() const{
    return this->daysToExpireLog;
}

int Conf::getUpdateTries() const{
    return this->updateTries;
}

int Conf::getUpdateTimeOut() const{
    return this->updateTimeOut;
}

/*
 * size Functions
 * check if variables were setted.\
 */

int Conf::sizeDataDir() {
    return this->dataDir.size();
}

int Conf::sizeLogDir() {
    return this->logDir.size();
}

int Conf::sizeProxyConf() {
    return this->proxyConf.size();
}

int Conf::sizeWebService() {
    return this->webService.size();
}

int Conf::sizeDelay() {
    ostringstream convert; 
    convert << this->delay;
    return (convert.str()).size();
}

int Conf::sizeDaysToExpireLOg() {
    ostringstream convert; 
    convert << this->daysToExpireLog;
    return (convert.str()).size();
}

int Conf::sizeUpdateTries() {
    ostringstream convert; 
    convert << this->updateTries;
    return (convert.str()).size();
}

int Conf::sizeUpdateTimeOut() {
    ostringstream convert; 
    convert << this->updateTimeOut;
    return (convert.str()).size();
}

/*
 * setConf Function
 * If variables from config file setted,
 * then set the conf object
 */

bool setConf(Conf& conf, Agent agent, map<string,string>& content){
    if(content.count("DATADIR") == 0 || 
        content.count("LOGDIR") == 0 || 
        content.count("PROXYCONF") == 0 ||
        content.count("WEBSERVICE") == 0|| 
        content.count("DELAY") == 0 ||
        content.count("DAYSTOEXPIRELOG") == 0 ||
        content.count("UPDATETRIES") == 0 ||
        content.count("UPDATETIMEOUT") == 0 )
            return false;

    /* 
     * agent.getPrefix() contains the absolute path of the agent. 
     */

    string dataDir = agent.getPrefix() + content["DATADIR"];
    string dataLog = agent.getPrefix() + content["LOGDIR"];
    string proxyConf = agent.getPrefix() + content["PROXYCONF"];
    string webService = content["WEBSERVICE"];
	
    int delay = atoi(content["DELAY"].c_str()); /* 24 hours */
    int daysToExpireLog = atoi(content["DAYSTOEXPIRELOG"].c_str()); /* 1 month */
    int updateTries = atoi(content["UPDATETRIES"].c_str());
    int updateTimeOut = atoi(content["UPDATETIMEOUT"].c_str());

    conf.setDataDir(dataDir);
    conf.setLogDir(dataLog);
    conf.setProxyConf(proxyConf);
    conf.setWebService(webService);

    conf.setDelay(delay);
    conf.setDaysToExpireLog(daysToExpireLog);
    conf.setUpdateTries(updateTries);
    conf.setUpdatTimeOut(updateTimeOut);

    return true;
}

Conf datasid_conf(Agent agent){
    map<string, string> content;
    string confFile = agent.getPrefix() + "/conf/datasid-conf.json";
    Conf conf;

    if(readfile(confFile, content)){
        if(!setConf(conf, agent, content)){
            cout << __DATE__ << " ERROR: to parse variables from config file." << endl;
            exit(1);
        }
    }else {
        cout << __DATE__ << " ERROR: config file not found." << endl;
        exit(1);
    }

    return conf;
}