/* Copyright (C) 2004-2016 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of datasid
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 */
#include "../../include/agent/linux/datasid-parse.h"
/* 
 * Erases leading tabs, leading spaces; trailing tabs, trailing spaces. 
 */

void trim (string& str){
    int i = 0;

    while (i < (int) str.length() && (str[i] == ' ' || str[i] == '\t'))
        ++i;

    if (i > 0)
        str.erase (0, i);

    int j = i = str.length();

    while (i > 0 && (str[i - 1] == ' ' || str[i - 1] == '\t'))
        --i;

    if (i < j)
        str.erase (i, j);
}

/* 
 * Erases tabs and spaces between the variable's name and its value. 
 */

void normalize (string & str){
    trim (str);

    size_t i, j;
    i = j = 0;

    while (i < str.length()) {
        if (str[i] == ' ' || str[i] == '\t') {
            j = i + 1;

            while (j < str.length() && (str[j] == ' ' || str[j] == '\t'))
                ++j;

            if (j == i)
                str.erase (i, 1);
            else
                str.erase (i, j - i);
        }
        else 
            ++i;
    }
}

/* 
 * Check if a line consists only of spaces and tabs 
 */

bool spaceonly (const string& line){
    for (int i = 0, j = line.length(); i < j; ++i)
        if (line[i] != ' ' && line[i] != '\t')
            return false;
    return true;
}

/* 
 * Check if a line is valid 
 */

bool isvalid (string& line) {
    normalize (line);
    int i = 0;

    if (line[i] == '[') {
        int j = line.find_last_of (']');

        if (j == std::string::npos)
            return false;

        if (j - i == 1)
            return false;
    }

    else if (line[i] == ';' || line[i] == '#' || (line[i] == '/' && line[i + 1] == '/'))
        return false;

    else if (line[i] == '=' || line[i] == ']' || line[i] == '{' || line[i] == '}')
        return false;

    else if (line[i] = '"'){
        int j = line.find_last_of (':');
        if (j == i || j == -1)
            return false;

        if (j == line.length())
            return false;
    }

    return true;
}

void parse ( string& line, map<string, string>& content){
    locale loc;
    // if the line is a variable + value
    if (line[0] == '"') {

        string name = line.substr (1, line.find (':')-2);
        for(int i = 0; i < name.length(); i++)
            name[i] = toupper(name[i], loc);

        string value = line.substr (line.find (':')+2); 
        value = value.substr(0,value.find ('"'));

        content[name] = value;
    }
}

bool readfile (const string file, map<string,string>& content) {
    content.clear();

    ifstream config (file.c_str(), ios::in);

    if (!config.is_open())
        return false;

    string section;
    string buffer;

    while (getline (config, buffer, '\n')){
        if (!spaceonly (buffer))
            if (isvalid (buffer))
                parse ( buffer, content);
    }
    
    return true;
}