/* Copyright (C) 2004-2016 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of datasid
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 */
#include "../../include/agent/linux/datasid-conf.h"
#include "../../include/agent/linux/datasid-parse.h"
#include "../../include/agent/linux/datasid-proxy.h"
#include "../../include/agent/linux/datasid-agent.h"

using namespace std;

/*
 * set Functions
 */

void Proxy::setHost(string h){
    this->host = h;
}

void Proxy::setPort(string p){
    this->port = p;
}

void Proxy::setUser(string u){
    this->user = u;
}

void Proxy::setPassword(string p){
    this->password = p;
}

/*
 * get Functions
 */

string Proxy::getHost() const{
    return this->host;
}

string Proxy::getPort() const{
    return this->port;
}

string Proxy::getUser() const{
    return this->user;
}

string Proxy::getPassword() const{
    return this->password;
}


bool setProxy(Proxy& proxy, map<string,string>& content){
    if(content.count("PROXYHOST") == 0 || 
        content.count("PROXYPORT") == 0 || 
        content.count("PROXYUSER") == 0 ||
        content.count("PROXYPASSWORD") == 0)
            return false;

    proxy.setHost(content["PROXYHOST"] );
    proxy.setPort(content["PROXYPORT"]);
    proxy.setUser(content["PROXYUSER"]);
    proxy.setPassword(content["PROXYPASSWORD"]);

    return true;
}

/*
 * I suppose there are a file "proxy" that contain
 * host, port, user and password variables.
 */

Proxy datasid_proxy(Conf conf){
    Proxy proxy;
    map<string,string> content;
    string proxyFile = conf.getProxyConf();

    if(readfile(proxyFile, content)){
        if(!setProxy(proxy, content)){
            cout << __DATE__ << " ERROR: to parse variables from proxy file." << endl;
            exit(1);
        }
    } else
        cout << __DATE__ << " WARNING: proxy file not found." << endl;
	
    return proxy;
}